__metaclass__ = type

__all__ = ['IKeyValuePair',
           'IPairSet',
           'KeyValuePair',
           'PairSet']

from zope.interface import implements
from zope.schema import Text
from zope.location.interfaces import ILocation

from lazr.restful.declarations import (
    collection_default_content, export_as_webservice_collection,
    export_as_webservice_entry, export_operation_as,
    export_read_operation, exported, operation_for_version,
    operation_parameters, operation_removed_in_version)

# Our implementations of these classes can be based on the
# implementations from the WSGI example.
from lazr.restful.example.wsgi.resources import (
    PairSet as BasicPairSet, KeyValuePair as BasicKeyValuePair)

# Our interfaces _will_ diverge from the WSGI example interfaces, so
# define them separately.
class IKeyValuePair(ILocation):
    export_as_webservice_entry()
    key = exported(Text(title=u"The key"))
    value = exported(Text(title=u"The value"))


class IPairSet(ILocation):
    export_as_webservice_collection(IKeyValuePair)

    # In versions 2.0 and 3.0, the collection of key-value pairs
    # includes all pairs.
    @collection_default_content("2.0")
    def getPairs():
        """Return the key-value pairs."""

    # Before 2.0, it only includes pairs whose values are not None.
    @collection_default_content('beta')
    def getNonEmptyPairs():
        """Return the key-value pairs that don't map to None."""

    def get(request, name):
        """Retrieve a key-value pair by its key."""

    # This operation is not published in trunk.
    @operation_removed_in_version('trunk')
    # In 3.0, it's published as 'by_value'
    @export_operation_as('by_value')
    @operation_for_version('3.0')
    # In 1.0 and 2.0, it's published as 'byValue'
    @export_operation_as('byValue')
    @operation_parameters(value=Text())
    @export_read_operation()
    @operation_for_version('1.0')
    # This operation is not published in versions earlier than 1.0.
    def find_for_value(value):
        """Find key-value pairs that have the given value."""


class PairSet(BasicPairSet):
    implements(IPairSet)

    def find_for_value(self, value):
        return [pair for pair in self.pairs if value == pair.value]

    def getNonEmptyPairs(self):
        return [pair for pair in self.pairs if pair.value is not None]

class KeyValuePair(BasicKeyValuePair):
    implements(IKeyValuePair)
